C
c     Description
c
c     Create an azimuthally-averaged, radial spectrum from an MMTF
c     image.  Optionally, turn this spectrum into an image of the
c     background and subtract it from the original image (to, e.g.,
c     subtract the sky from data).
c
c     Authors
c
c     Original software by Ben Weiner, adapted for MMTF use by Ben
c     Weiner and David Rupke.
c
c     History
c
c     08apr02     DSNR  new gravity angle computation/output
c     08feb21     DSNR  added implicit none
c                       bin size for med. smoothing now specified in A
c                       raw data overplotted during median smoothing
c     07oct-nov   DSNR  major re-write
c     May 2005    BW    revised for mosaic images
c     circa 1992  BW    created
c
c
C=============
C Definitions
C=============
c
      implicit none
c
      integer nzones,maxzone,maxpts,numberchips,maxccdx,biwtmeth,npkpts
c
C     Number of radial bins.  1000 bins sample every 0.1 A at 6600 A,
C     assuming an aperture of 4000 pixels radius and the IMACS f/2 focal
C     length of 24000 pixels.  This is about 3% of the minimum
C     achievable FWHM at 6600 A with MMTF.  The uncertainty in the
C     biweight estimate scales roughly as 1/sqrt(8092^2/NZONES) ~
C     sqrt(NZONES)/8092.
      parameter(nzones=1000)
c
C     Maximum number of radial bins is maxzone (if we ever want to allow
C     it to vary) and maximum number of pixels in a bin is maxpts.
C     Maximum number of elements in an array is 2^29, so maximum value
C     of maxzone is 2^29/NZONES.
c
C     For some reason, the biwgt() call slows to a crawl on my Mac
C     laptop if maxpts is too large (say ~200-250,000 points or more).
C     So, use something smaller than the maximum possible value of
C     maxpts.
c
c     For NZONES, the approximate number of points per radial bin in an
c     8092 x 8092 array is 8092^2/NZONES.  We'll factor this up by 10%
c     just to be safe.
c
c     parameter (maxzone=NZONES,maxpts=int(536870912/NZONES))
      parameter(maxzone=NZONES,maxpts=int(8092*8092/NZONES*1.1))
c     Number of IMACS CCDs
      parameter(numberchips=8)
C     Maximum number of CCD columns (to allow for overscan).
      parameter (maxccdx=2400)
c     Use biweight or something else?
c     1 = biweight
c     2 = median
c     3 = sigma-clipped median
      parameter(biwtmeth=1)
c     Number of points to use for peak-finding algorithm
      parameter(npkpts=15)

      real data(maxccdx)
      real skydata(maxzone,maxpts),tempdata(maxpts)
      real sum(maxzone),sumsq(maxzone),ave(maxzone)
      real err(maxzone),rms(maxzone)
      real yetop(maxzone),yebot(maxzone)
      real wavemean(maxzone),smspec(maxzone)
      real wave(maxzone),flux(maxzone),ferr(maxzone)
      real wavesort(maxzone),fluxsort(maxzone)
      real yplot(maxzone)
      real skybiwt(maxzone),skybisc(maxzone),skybierr(maxzone)
      real skybirms(maxzone),skybiraw(maxzone)
      real fluxpkarr(npkpts),wavepkarr(npkpts)
      real vgtfit(5),fiterr(5),dv(5),fitguess(2),plotpkw(2),plotpkf(2)
      real wave0,ff,chisq,bad,den
      real sgv,slv,sg,sl
      real w1,w2,restwave,wavemin,wavemax,wavepk,wavepkmed,wavepkmad
      real fluxl,fr,fluxpk,fluxpkmed,fluxpkmad
      real biw,biscale,bitune,fact,factccd,chiptofpfac,majtick
      real xplotmin,xplotmax,yplotmin,yplotmax
      real yerrplotmin,yerrplotmax,yrmsplotmin,yrmsplotmax
      real chopfive,bottomfive,topfive,smbina
      real ar,ax,ay,axfp,ayfp,rx,ry,rxfp,ryfp,dx,dy
      real arsq,aradsq,rmaxsq,rrsq
      real xtrue,ytrue,xfp,yfp
c      real gravangle,rotangle,telelev,sitelat,decd
      real gravangle,gravmax,gravmin

      integer i,j,iord,imin,imax,line,col,ichip,nring,ier,ierr
      integer imptr(numberchips),imptrnew(numberchips)
      integer nxpix(numberchips),nypix(numberchips)
      integer nxpixsky(numberchips),nypixsky(numberchips)
      integer num(maxzone),imsize(7),itemp,imdim,imtype
      integer numchips,nxbin,nybin,nccdxmax,nccdymax,nxtemp,nytemp
      integer ir,indexc,indexlw,flag,pgstat,psplot,nargs
      integer doerr,dofast,dofit,domeds,dosub,inspec
      integer binstep,nbinsteps,nbinpct,npixsamp,replot,ios
      integer xfine,yfine,zfine,zcoarse,ilen,ilen2,ilen3,kflag
      integer ibottomfive,itopfive,chopzones,iwavepk,iwavepkmed,smbin
      integer nchipapcen,nchiprcen

      character*80 filename,subtfile,datfile,psfile,inskyfile
      character*80 xlabel,toplabel1,toplabel2,arg,peaklabel1
      character*80 time,date,filter,cname,dummy
      character answer*3,bell*1

c     PGPLOT subroutine declaration.
      integer pgopen

C=========================
C Set initial parameters 
C=========================

C     Is this calibration data?
      dofast=0
C     Invoke interactive line fit?
      dofit=0
C     Subtract sky?
      dosub=0
c     Do median smoothing?
      domeds=0
C     Have we re-plotted?
      replot=0
C     Plot error spectra on output?
      doerr=0
c     Use input spectrum?
      inspec=0
      
C     Get command-line arguments.
      nargs = iargc()
      if (nargs .lt. 2) then
         write (*,'(3(a),7(/,a))') "Usage: ring",
     $        " (input file basename; e.g., ccd1234)",
     $        " (central wavelength)","   Optional flags:",
     $        "      --sub : Subtract computed background",
     $        "      --fit : Interactive Voigt profile fit",
     $        "      --ms [A] : Median smooth, window size [Angstroms]",
     $        "      --err : Plot error spectra on output",
     $        "      --fast : Decrease sampling to increase speed",
     $        "      --spec [filename] : Take sky from input spectrum"
         stop
      end if
      call getarg(1,arg)
      read (arg,*) filename
      call getarg(2,arg)
      read (arg,*) wave0
      if (nargs .gt. 2) then
         do i=3,nargs
            call getarg(i,arg)
            if (arg .eq. '--fast') then
               dofast = 1
            end if
            if (arg .eq. '--fit') then
               dofit = 1
            end if
            if (arg .eq. '--sub') then
               dosub = 1
            end if
            if (arg .eq. '--err') then
               doerr = 1
            end if
            if (arg .eq. '--ms') then
               domeds = 1
               if (i .eq. nargs) then
                  write(*,'(3(a))') 'RING: Must specify size '
     $                 ,'of bin for median smoothing after switch '
     $                 ,'(e.g., ring ... --ms 5).'
                  stop
               else
                  call getarg(i+1,arg)
                  read (arg,*) smbina
               end if
            end if
            if (arg .eq. '--spec') then
               inspec = 1
               if (i .eq. nargs) then
                  write(*,'(3(a))') 'RING: Must specify filename '
     $                 ,'for input sky spectrum '
     $                 ,'(e.g., ring ... --inspec ccd0001.rings.mod).'
                  stop
               else
                  call getarg(i+1,arg)
                  read (arg,*) inskyfile
               end if
            end if
         end do
      end if            

C     Reject pixels below this value in ring averaging and when
C     subtracting sky from input image.
      bad = -1.0e3

C     Number of IMACS CCD chips.
      numchips = numberchips

C     Biweight tuning constant
      bitune = 6

C     Input file containing optical axis.
      cname = 'optaxis.dat'
      open (10,file=cname,status="old",iostat=ios)
      if (ios .ne. 0) then 
         write(*,'(2(a))')
     $        "RING: Error opening file containing optical axis: ",cname
         stop
      end if
      read (10,*) dummy
      read (10,*) dummy
      read (10,*) nchipapcen,ax,ay
      if (nchipapcen.gt.numchips) then
         write(*,'(2(a))')
     $        "RING: Chip with optical axis out of range."
         stop
      end if         
C     Remove option for aperture center and optical axis to be different
C     -- irrelevant for MMTF, which cannot be tilted.
      nchiprcen = nchipapcen
      rx = ax
      ry = ay
C     Aperture radius in focal plane.  Could be ~50-100 pixels larger
C     before aperture hits edge of chip 2, and 400-500 pixels larger
C     before aperture hits aperture mask.  However, begin to run into
C     guider probe vignetting on edge chips beyond ~4000 pixels, and to
C     lose significant #s of pixels at very large radii, increasing the
C     noise.
      ar = 4000
      arsq = ar * ar

C     Close input file.
      close(10)

C     Camera focal length, in pixels.  Used to be read from input file.
C     Hardwired for current IMACS setup.
      ff = 23600

C     Initial guesses for line fit: Gaussian and Lorentzian FWHMs, in
C     km/s.  Used to be read from input file.
      sgv = 10
      slv = 10

C     Intervals between major tick marks on plots, in wavelength units.
      majtick=20.

C===================
C Read input image
C===================

c     Call a subroutine that opens the mosaic images and returns an
c     array of image pointers.  (This relies on having enough pointers
c     to keep all images open at same time.)

 40   continue
c      write(*,'("Enter image basename: ",$)')
      call openmosaicimage(filename,numchips,imptr,ierr)
      if (ierr.ne.0) then
         write(*,*) "RING: Exiting because image not found."
         stop
      end if

c     Get information from header
c     - Fabry-Perot parameters: X_fine, Y_fine, Z_fine, and Z_coarse
c     - UT time, date
c     - Filter
c     - Rotator offset angle, telescope elevation, LCO latitude, and 
c       pointing dec to compute gravity angle.
      call imgkwi(imptr(1),"MMTF-X",xfine,ier)
      call imgkwi(imptr(1),"MMTF-Y",yfine,ier)
      call imgkwi(imptr(1),"MMTF-Z",zfine,ier)
      call imgkwi(imptr(1),"MMTF-CZ",zcoarse,ier)
      call imgkwc(imptr(1),"UT-TIME",time,ier)
      call imgkwc(imptr(1),"UT-DATE",date,ier)
      call imgkwc(imptr(1),"FILTER",filter,ier)
c      call imgkwr(imptr(1),"ROTANGLE",rotangle,ier)
c      call imgkwr(imptr(1),"TEL-ELEV",telelev,ier)
c      call imgkwr(imptr(1),"SITELAT",sitelat,ier)
c      call imgkwr(imptr(1),"DEC-D",decd,ier)
      call imgkwr(imptr(1),"GRAV-MIN",gravmin,ier)
      call imgkwr(imptr(1),"GRAV-MAX",gravmax,ier)

C========================
C Initialize some things
C========================

c     Determine chip size to compute binning.  This is for IMACS chips
c     which are 2048x4096 and have overscan in both x and y but might be
c     binned 1x1, 2x2, 4x4, or 8x8, and we want to trim the overscan
c     off.
      call imgsiz (imptr(1), imsize, imdim, imtype, ier)
      nxtemp = imsize(1)
      nytemp = imsize(2)
      if (nxtemp .ge. 230 .and. nxtemp .le. 370) then
         nxbin = 8
         nybin = 8
         nccdxmax = 256
         nccdymax = 512
      else if (nxtemp .ge. 500 .and. nxtemp .le. 700) then
         nxbin = 4
         nybin = 4
         nccdxmax = 512
         nccdymax = 1024
      else if (nxtemp .ge. 1000 .and. nxtemp .le. 1600) then
         nxbin = 2
         nybin = 2
         nccdxmax = 1024
         nccdymax = 2048
      else if (nxtemp .ge. 2000 .and. nxtemp .le. 4000) then
         nxbin = 1
         nybin = 1
         nccdxmax = 2048
         nccdymax = 4096
      end if
      write(*,'("Chip binning is: ",i1," x ",i1)') nxbin,nybin

c
c     Compute gravity angle
c
c     OLD: We're assuming rotangle is the NASW (= Nasmyth West) rotator
c     angle.  See
c
c     http://www.lco.cl/telescopes-information/magellan/instruments-1/observing-catalogs-1
c     
c     Remember that when telescope is at zenith and IMACS is "parked,"
c     rotangle = -46.15, and that its limits are +/-180 degrees.  The
c     second term below accounts for the movement of IMACS as the
c     telescope changes in elevation (while the rotator offset is kept
c     fixed) -- the direction depends on whether the target is to the
c     north or south.  The rotangle term takes care of changes in
c     rotator offset to change the sky position angle (which is constant
c     otherwise as IMACS slews/tracks).
c
c      if (decd.gt.sitelat) then
c         gravangle = rotangle - (90. - telelev)
c      else
c         gravangle = rotangle + (90 - telelev)
c      end if
c      if (gravangle.gt.180.) then
c         gravangle = gravangle - 360.
c      else if (gravangle.lt.-180.) then
c         gravangle = gravangle + 360.
c      end if
c
c     NEW: Just using gravity angle in header -- average of GRAV-MIN and
c     GRAV-MAX.  At IMACS "park" position (usual stowing position), have
c     GRAV-XXX ~ 0 and ROTATRE ~ -46.15.  At IMACS "home" (ready to
c     track with telescope), have GRAV-XXX ~ -46.15 and ROTATORE ~ 0.
c     So they're moving in different directions.
c
      gravangle = (gravmin + gravmax) / 2.

c     A factor to convert between CCD and focal plane units.
      chiptofpfac = real(nxbin)
c     'chiptofp()' returns focal plane positions in unbinned CCD units,
c     rather than focal plane units.
      call chiptofp(nchiprcen,rx,ry,rxfp,ryfp)
      call chiptofp(nchipapcen,ax,ay,axfp,ayfp)
c     difference between aperture center and optical axis
      dx = abs(rxfp-axfp)
      dy = abs(ryfp-ayfp)
c     'arsq' has units of unbinned CCD pixels.
      rmaxsq = arsq + dx*dx + dy*dy

c     fact is 1/(width of radial bin^2), where the radial bin width is
c     in unbinned pixels, in the focal plane
 100  fact = NZONES / rmaxsq
c     factccd is the same as fact, but the zone width is in binned
c     pixels
      factccd = NZONES / (rmaxsq/chiptofpfac**2)

C=====================================
C Read input sky spectrum, if present
C=====================================

 50   if (inspec.eq.1) then

         open (20,file=inskyfile,status='unknown',iostat=ios)
         if (ios .ne. 0) then 
            write(*,'(3(a))')
     $           "RING: Error opening file containing sky spectrum: "
     $           ,inskyfile,".  Computing new spectrum instead."
            go to 80
         end if
         write(*,'("Opening input sky spectrum: ",a30)') inskyfile
c     Skip header lines.
         read(20,*) dummy
         read(20,*) dummy
         read(20,*) dummy
         read(20,*) dummy
         read(20,*) dummy
c     Read data row by row.
 55      read(20,*,iostat=ios) i,wavemean(i),num(i),skybiwt(i),skybisc(i
     $        ),skybierr(i),ave(i)
         yetop(i) = skybiwt(i) + skybierr(i)
         yebot(i) = skybiwt(i) - skybierr(i)
         if (i .gt. 1) then
            skybirms(i) = skybiwt(i) - skybiwt(i-1)
         else
            skybirms(i) = 0
         endif
C     Check for end-of-file, otherwise keep reading.
         if (ios.ne.0) then
            close(20)
            go to 130
         else
            go to 55
         end if

      end if

C=====================================
C Initialize a few additional things
C=====================================

C     Sample every 'npixsamp' pixels along each axis.
 80   if (dofast .eq. 0) then
         npixsamp = 1
      else
         if (nxbin .eq. 8) then
            npixsamp = 1
         else if (nxbin .eq. 4) then 
            npixsamp = 2
         else if (nxbin .eq. 2) then 
            npixsamp = 8
         else
            npixsamp = 4
         end if
      end if
      write (*,'(a,i1,a)') "Sampling every ",npixsamp
     $     ," pixel(s) on each axis."
      write (*,'(a,i4,a)') "Using ",NZONES ," radial bins."

C     Cycle through radial bins to compute wavelengths.
      do i=1,maxzone
         sum(i) = 0.0
         sumsq(i) = 0.0
         num(i) = 0
C     We ignore Z-dependence, since the central wavelength is given.
         w1 = wave0 / sqrt(1. + (i-1.)/fact/ff/ff)
         w2 = wave0 / sqrt(1. +  i    /fact/ff/ff)
         wavemean(i) = (w1 + w2) /2.0 - wave0
      end do

C=================================
C Separate data into radial bins
C=================================

 110  write(*,'("Putting data in radial bins: Chip",$)')

c     Loop through chips
      do ichip = 1,numchips

         write(*,'(i2,$)') ,ichip
         
         call imgsiz (imptr(ichip), imsize, imdim, imtype, ier)
         nxpix(ichip) = imsize(1)
         nypix(ichip) = imsize(2)
c     Ignore overscan in sky calculation
         nxpixsky(ichip) = min(nccdxmax,nxpix(ichip))
         nypixsky(ichip) = min(nccdymax,nypix(ichip))

         do line=1,nypixsky(ichip),npixsamp
            call imgl2r (imptr(ichip), data, line, ier)
            do col=1,nxpixsky(ichip),npixsamp
               if (data(col) .gt. bad) then
c     The pixel center in unbinned coords
                  xtrue = nxbin*(col-0.5)+0.5
                  ytrue = nybin*(line-0.5)+0.5
c     The pixel center in the focal plane
                  call chiptofp(ichip,xtrue,ytrue,xfp,yfp)
c     Distance from the aperture center, in the focal plane
                  aradsq = (axfp-xfp)**2 + (ayfp-yfp)**2
                  if (aradsq .lt. rmaxsq) then
c     Distance from the optical axis, in the focal plane
                     rrsq = (rxfp-xfp)**2 + (ryfp-yfp)**2
c     Radial bin number
                     ir = 1 + int(rrsq * fact)
                     if (ir .le. NZONES) then
                        sum(ir) = sum(ir) + data(col)
                        sumsq(ir) = sumsq(ir) + data(col) * data(col)
                        num(ir) = num(ir) + 1
                        if (num(ir) .gt. maxpts) then
                           write(*,'(a,i6,a,i4,a,i4,a)') "More than "
     $                          ,maxpts," points in radial bin #",ir
     $                          ," out of ",NZONES,"."
                           write(*,'(a)')
     $ "Increase the number of bins or decrease the sampling rate."
                           go to 80
                        end if
                        skydata(ir,num(ir)) = data(col)
                     end if
                  endif
               end if
            end do
         end do
      end do
      write(*,*)

C=====================================
C Compute background value and error
C=====================================

 120  write(*,'("Doing azimuthal averaging:",$)')
      
      binstep = NZONES / 10
      nbinsteps = binstep
      nbinpct = 10
      
      do i=1,NZONES

         if (i .ge. nbinsteps) then
            if (nbinpct .eq. 100) then 
               write(*,'(i4,a,$)') nbinpct,"%"
            else
               write(*,'(i3,a,$)') nbinpct,"%"
            end if
            nbinsteps = nbinsteps + binstep
            nbinpct = nbinpct + 10
         end if

c     Compute mean and standard error.
c     'ave(i)' is the mean in the i'th ring.
c     'err(i)' is the standard error in the mean.
         if (num(i) .ge. 1) then
            ave(i) = sum(i) / num(i)
            rms(i) = sqrt((sumsq(i)/num(i)) - ave(i)*ave(i))
         else
            ave(i) = bad
            rms(i) = 1.0e6
         end if
         den = num(i) - 1.0
         if (den .lt. 1.0) den = 1.0
         err(i) = rms(i) / sqrt(den)

c     Compute bi-weight (or median, or sigma-clipped mean).
c     'skybiwt(i)' is the biweight.
c     'skybierr(i)' is the standard error in the biweight.

         tempdata(:) = skydata(i,:)
         itemp = num(i)
         if (num(i) .ge. 1) then
            if (biwtmeth .eq. 2) then
               call domedian(tempdata,itemp,biw,biscale)
            else if (biwtmeth .eq. 3) then
               call dosigclip(tempdata,itemp,ave(i),rms(i),2.5,biw
     $              ,biscale)
            else
               call biwgt(tempdata,itemp,biw,biscale,bitune)
            end if
         
            skybiwt(i) = biw
            skybisc(i) = biscale
            skybierr(i) = biscale / sqrt(den)
            if (i .gt. 1) then
               skybirms(i) = skybiwt(i) - skybiwt(i-1)
            else
               skybirms(i) = 0
            endif
         else
            skybiwt(i) = bad
            skybisc(i) = 1.0e6
            skybierr(i) = 1.0e6
            skybirms(i) = 1.0e6
         end if
         yetop(i) = skybiwt(i) + skybierr(i)
         yebot(i) = skybiwt(i) - skybierr(i)
      end do
      write(*,*)


c
C     Jump to here if we used an input sky spectrum.
c
 130  continue

C     Median smooth spectrum?
      if (domeds.eq.1) then
c     Convert binning parameter from A to bins.  Assumes A/bin is
c     constant, which is true if our bins are bins of constant 1/r^2.
         smbin = int(smbina / abs(wavemean(2)-wavemean(3)))
         if (smbin.eq.0) smbin = 1
         write(*,'(a,f5.1,a,i4,a)')
     $        "Median smoothing with window size of ",smbina," A, or "
     $        ,smbin," bins."
         if (smbin.gt.1) then
            call medsmooth(NZONES,skybiwt,smbin,smspec)
            skybiraw=skybiwt
            skybiwt=smspec
         endif
      end if

C========================
C Peak-finding algorithm
C========================

C     Trim edges of spectrum in case of funny business.
C     Get wavelength bordering top 5% and bottom 5% of wavelength space.
C     Note that wavemean(1) is the high wavelength!
      chopfive = NZONES/20
      ibottomfive = NZONES-chopfive
      itopfive = chopfive
C     Number of points in chopped window.
      chopzones = ibottomfive - itopfive + 1
C     Chopped data.
      wavesort=wavemean(itopfive:ibottomfive)
      fluxsort=skybiwt(itopfive:ibottomfive)
C     Actual peak in flux.
      fluxpk=0
      do i=1,chopzones
         fluxpk = max(fluxsort(i),fluxpk)
      end do
C     Sort data in descending flux order.
      kflag=-2
      call ssort(fluxsort,wavesort,chopzones,kflag)
C     Find highest few points in flux space
      wavepkarr=wavesort(1:npkpts)
C     Get median of these points.
      call medmad(wavepkarr,npkpts,wavepkmed,wavepkmad)
C     Get index of the median-filtered and actual peak wavelengths.
      do i=1,NZONES
         if (wavemean(i).ge.wavepkmed) iwavepkmed=i
         if (skybiwt(i).eq.fluxpk) iwavepk=i
      end do
C     Find median flux at this radius by taking a small window (+/- 5
C     zones).
      fluxpkarr=skybiwt(iwavepk-5:iwavepk+5)
      call medmad(fluxpkarr,11,fluxpkmed,fluxpkmad)
C     Wavelength of actual peak flux.
      wavepk = wavemean(iwavepk)

C======================
C Print to output file
C======================

C     Open output file only if there was no input spectrum.  It will
C     contain the averaged background spectrum.

      if (inspec.eq.0) then

 140     ilen = index(filename,' ')-1
         datfile = filename(1:ilen)//'.rings'
         open (30,file=datfile,status='unknown')
         
         write(*,'("Writing output spectrum: ",a30)') datfile
c     Header information.
         write(30,'(a10,3(a8),a10,a15)') "#  wl(r=0)","X_fine","Y_fine"
     $        ,"Z_fine","Z_coarse","Gravity Angle"
         ilen=index(date,' ')-1
         ilen2=index(time,' ')-1
         ilen3=index(filter,' ')-1
         write(30,'(f10.2,3(i8),i10,f15.2)') wave0,xfine,yfine,zfine
     $        ,zcoarse,gravangle
         write(30,'(a12,a10,a20,a10,a10)') "#    UT-date","UT-start"
     $        ,"Filter","PeakWave","PeakFlux"
         write(30,'(a12,a10,a20,f10.2,f10.0)') date(1:ilen),time(1:ilen2
     $        ),filter(1:ilen3),wavepkmed,fluxpkmed
c     Data.
         write(30,'(a6,a14,a8,4(a10))') "# ring","wl(r)-wl(r=0)","#pts"
     $        ,"biweight" ,"bi-scale","bi-err","mean"
         do i=1,NZONES
            write(30,'(i6,f14.2,i8,4(f10.2))') i,wavemean(i),num(i)
     $           ,skybiwt(i),skybisc(i),skybierr(i),ave(i)
         end do
         close(30,status="keep")

      end if

C==============
C Plot results
C==============

C     Compute maximum and minimum background and background error
C     values.
      yplotmin = yebot(1)
      yplotmax = yetop(1)
      yerrplotmin = skybierr(1)
      yerrplotmax = skybierr(1)
      yrmsplotmin = skybirms(1)
      yrmsplotmax = skybirms(1)
      do i=1,NZONES
         if (skybiwt(i) .gt. -1000.) then
            yplotmin = amin1(yplotmin,yebot(i))
            yplotmax = amax1(yplotmax,yetop(i))
            yerrplotmin = amin1(yerrplotmin,skybierr(i))
            yerrplotmax = amax1(yerrplotmax,skybierr(i))
            yrmsplotmin = amin1(yrmsplotmin,skybirms(i))
            yrmsplotmax = amax1(yrmsplotmax,skybirms(i))
         end if
      end do

c     Plot results to screen

      psplot = 0
C     Look for proper screen device
 185  pgstat = pgopen('/XS')
      if (pgstat .le. 0) then
         write(*,*) "/XS device unavailable; trying /AQT."
         pgstat = pgopen('/AQT')
         if (pgstat .le. 0) then
            write(*,*) "/AQT device unavailable; querying."
            pgstat = pgopen('?')
         end if
      end if
C     Initialize.
 190  call pgeras()
      call pgscf(2)
      call pgslw(2)
      call pgask(.false.)
      xplotmin = wavemean(NZONES) 
      xplotmax = wavemean(1)
C     Plot background only or background plus error spectra?
      if (doerr.eq.0) then
         call pgsvp(0.1,0.9,0.1,0.9)
      else
         call pgsvp(0.1,0.9,0.3,0.9)
      end if
      call pgswin(xplotmin,xplotmax,yplotmin,yplotmax)
c     overplot raw data if we median smoothed
      if (domeds.eq.1) then
         call pgqci(indexc)
         call pgqci(indexlw)
         call pgsci(4)
         if (psplot.eq.1) call pgsls(4)
         call pgline(NZONES,wavemean,skybiraw)
         if (psplot.eq.1) call pgsls(1)
         call pgsci(indexc)
         call pgslw(4)
         call pgline(NZONES,wavemean,skybiwt)
         call pgslw(indexlw)
      else
         call pgline (NZONES,wavemean,skybiwt)
      endif
C     Plot line through peak
      call pgsls(2)
      plotpkf = (/ 0, 1000000000 /)
      plotpkw = (/ wavepkmed, wavepkmed /)
      call pgline (2, plotpkw, plotpkf)
      call pgsls(1)
C     Write top and axis labels.
      ilen = index(filename,' ')-1
      write(toplabel1,'(a,a11,a15,a10,f8.2,a26)') "UT ",date,time
     $     ,"Grav.Ang.=",gravangle,filename(1:ilen)
      write(toplabel2,'(a,i2,a,a27,3(a,i4))') "Z_c=",zcoarse
     $     ,"  Filter: ",filter,"X_f=",xfine,"  Y_f=",yfine ,"  Z_f="
     $     ,zfine
      write(peaklabel1,'(a,f6.1,a,a1)') "\gl\dpeak\u=",wavepkmed," ",197
      write(xlabel,'(a,a1,a)') "\gl(R) - \gl(R=0)  [",197,"]"
      call pgmtxt('T',1.9,0.5,0.5,toplabel1)
      call pgmtxt('T',0.7,0.5,0.5,toplabel2)
      call pgmtxt('LV',-0.9,0.95,0.,peaklabel1)
      if (doerr.eq.0) then
         call pgswin(wavemean(NZONES)-wavemean(1),0,yplotmin,yplotmax)
         call pgbox('BCNST',majtick,10,'BCNST',0,0)
         call pglabel (xlabel,'Counts / Pixel','')
      else
         call pgbox('BCST',majtick,10,'BCNST',0,0)
         call pglabel ('','Counts / Pixel','')
      end if
C     Are we plotting a line fit?
c     Plot fit in diff color.
      if (replot .eq. 1) then
         call pgqci(indexc)
         call pgsci(3)
         call pgline (NZONES,wavemean,yplot)
         call pgsci(indexc)
      end if
C     Plot errors and shifts in separate windows.
      if (doerr.ne.0) then
         call pgsvp(0.1,0.9,0.2,0.3)
         call pgswin(xplotmin,xplotmax,yerrplotmin,yerrplotmax)
         call pgbox('BCST',majtick,10,'BCMSTV',0,0)
         call pgline (NZONES,wavemean,skybierr)
         call pglabel ('','Error','')
         call pgsvp(0.1,0.9,0.1,0.2)
         call pgswin(wavemean(NZONES)-wavemean(1),0,yrmsplotmin
     $        ,yrmsplotmax)
         call pgbox('BCNST',majtick,10,'BCMSTV',0,0)
         call pgline (NZONES,wavemean,skybirms)
         call pglabel (xlabel,'Delta','')
      end if
      call pgend ()

c     Plot results to file

      if (psplot .eq. 0) then
C     Look for proper screen device
         ilen = index(filename,' ')-1
         psfile = filename(1:ilen)//'.rings.ps'
         pgstat = pgopen(psfile//'/PS')
         if (pgstat .le. 0) then
            write(*,*)
     $           "/PS device unavailable; not producing postscript."
         else
            write(*,'("Writing output plot: ",a30)') psfile
            psplot = 1
            go to 190
         end if
      end if

C     Is this a re-plot?
      if (replot .eq. 1) then
         write(*,'(a,$)') "Try another fit on this file? [n] "
         read (*,'(a)') answer
         if (answer(1:1) .ne. 'y') then
            dofit = 0
         end if
      end if

C=======================
C Interactive line fit
C=======================

 200  if (dofit .eq. 1) then
         
         write (*,210)
 210     format ('Enter min. and max. wavelength for Voigt fit: ',$)
         read (*,*) wavemin,wavemax

c     Fit "order."
         iord = 5

c     Convert sigmas into wavelength.
         restwave = (wavemin + wavemax) / 2.0
         sg = sgv * restwave / 3.0e+05
         sl = slv * restwave / 3.0e+05
      
c     Get the range of zones corresponding to the wavelength range.  Low
c     zone number (low i) corresponds to high wavelength.

 240     imin = 1
         imax = NZONES 
         do i=1,NZONES
            if (wavemean(i) .gt. wavemin) imax = i
            if (wavemean(i) .gt. wavemax) imin = i - 1
         end do
         
         nring = imax - imin + 1

         j=0
         do i=imin,imax
            if (num(i) .ge. 1) then
               j = j+1
               wave(j) = wavemean(i)
               flux(j) = skybiwt(i)
               tempdata(j) = flux(j)
c     Don't let error on flux be too small
               ferr(j) = max(skybierr(i),0.005*flux(j))
            end if
         end do


c     Do fitting.
         fitguess(1)=0.
         fitguess(2)=0.
 250     call evoigt (wave,flux,ferr,nring,sg,sl,iord,vgtfit,fiterr,
     $        chisq,flag,fitguess)

         if (flag .eq. 0) then

            write(*,'(a,f10.3)') "Reduced chi-squared: ",chisq
            write(*,260) "Parameter: ","Continuum","Flux","Wavelength"
     $           ,"Sig(Gau)","Sig(Lor)"
            write(*,260) "Units: ","[cts/pix]","[cts/pix*A]","[A]",
     $           "[A]","[A]"
            write (*,270) "Best fit: ",(vgtfit(i),i=1,5)
            write (*,280) "Error: ",   (fiterr(i),i=1,5)
 260        format (a12,a12,  a12,  a12,  a12,  a12  )
 270        format (a12,f12.0,f12.0,f12.2,f12.2,f12.2)
 280        format (a12,f12.0,f12.0,f12.2,f12.2,f12.2)

c     Compute the Voigt profile for re-plotting.
            do i=1,NZONES
               call voigt(wavemean(i),vgtfit,sg,yplot(i),dv)
            end do

c     Replot if we fitted.
            if (dofit .eq. 1) then
               write(*,*) "Re-plotting with line fit."
               replot = 1
               go to 185
            end if

         else

            write(*,*) "Fit crashed."
     $           ,"  Try again with another wavelength range."
            go to 200
            
         end if
         
      end if


C=====================================
C Subtract background from input file
C=====================================

 300  if (dosub .eq. 1) then

         ilen = index(filename,' ')-1
         subtfile = 'ss' // filename(1:ilen)
         ilen = index(subtfile,' ')-1
         write(*,'("Opening output FITS files: ",a,"c1-8.fits")')
     $        subtfile(1:ilen)
         call opncmosaicimage(subtfile,numchips,imptr,imptrnew,ierr)
         if (ierr .ne. 0) then
            write(*,*) "RING: Failed to open output image.  Exiting."
            go to 390
         end if

 310     write(*,'(a,$)')
     $        "Subtracting raw biweights from input image: Chip"



c     Loop through chips
         do ichip = 1,numchips
            
            write(*,'(i2,$)') ,ichip

c     Initialize chip sizes if they haven't been already
            if (inspec.eq.1) then
               call imgsiz (imptr(ichip), imsize, imdim, imtype, ier)
               nxpix(ichip) = imsize(1)
               nypix(ichip) = imsize(2)
            end if

            do line=1,nypix(ichip)
               call imgl2r (imptr(ichip),data,line,ier)
               do col=1,nxpix(ichip)
                  xtrue = nxbin*(col-0.5)+0.5
                  ytrue = nybin*(line-0.5)+0.5
                  call chiptofp(ichip,xtrue,ytrue,xfp,yfp)
                  aradsq = (axfp-xfp)**2 + (ayfp-yfp)**2
                  if (aradsq .lt. rmaxsq) then
c     Ignore nonlinearities and overscan when subtracting.
                     if (data(col) .gt. bad .AND. line .le. nccdymax
     $                    .AND.col .le. nccdxmax)then
                        rrsq = (rxfp-xfp)**2 + (ryfp-yfp)**2
                        ir = 1 + int(rrsq * fact)
                        fr = rrsq*fact - int(rrsq * fact)
C     Assume sky plateaus beyond last radial bin.
                        if (ir .ge. NZONES) then
                           fluxl = skybiwt(NZONES)
                        else
                           fluxl = (1.-fr)*skybiwt(ir) + fr*skybiwt(ir+1
     $                          )
                        end if
                        data(col) = data(col) - fluxl
                     end if
                  end if
               end do
               call impl2r (imptrnew(ichip),data,line,ier)
            end do
         end do
         write(*,*)

c     Close output image
         ilen = index(subtfile,' ')-1
         write(*,'("Writing output FITS files: ",a,"c1-8.fits")')
     $        subtfile(1:ilen)
         call closemosaicimage(numchips,imptrnew,ierr)

      end if
      
C=========
C THE END
C=========

c     Close input image
 390  call closemosaicimage(numchips,imptr,ierr)

      end
