C
c     Description
c
c     Compare peak wavelengths of the same calibration lamp emission
c     line taken at the same Z_fine but at different times.  Used for
c     measuring drift in the intercept of the wavelength - Z_fine
c     relation.  Requires radial spectra prodcued by RING routine.
c
c     Authors
c
c     David Rupke
c
c     History
c
c     07nov  DSNR  created
c

C=============
C Definitions
C=============

      implicit none

      integer MAXZONE
C     Number of radial bins.
      parameter(MAXZONE=1000)

      real wave(MAXZONE),flux(MAXZONE),ferr(MAXZONE)
      real wave0
      real plotpkw(2),plotpkf(2),diffpkmed
      real wavepkmed,fluxpkmed,newwavepkmed
      real xplotmin,xplotmax,yplotmin,yplotmax,majtick

      integer nimref,nimnew,nz,firstspec
      integer i,indexc,pgstat,psplot,nargs,replot,ios,domeds
      integer xfine,yfine,zfine,zfineref,zcoarse,ilen,ilen2,kflag

      character*80 name,newname,psfile,cname,dummy,arg,xlabel
      character*80 toplabel1,toplabel2,peaklabel0,peaklabel1,peaklabel2
      character*80 time,date,filter,filterref
      character    answer*3,radchg*7,achg*8,zchg*8

c     PGPLOT subroutine declaration.
      integer pgopen
      
C=========================
C Set initial parameters 
C=========================

c     Do median smoothing?
      domeds=0
      
C     Get command-line arguments.
 10   nargs = iargc()
      if (nargs .lt. 2) then
         write (*,'(3(a))') "Usage: ringdrift",
     $        " (reference file no., ccd prefix assumed)"
     $        ," (new file no.)"
         stop
      end if
      call getarg(1,arg)
      read (arg,*) nimref
      call getarg(2,arg)
      read (arg,*) nimnew

C     Intervals between major tick marks on plots, in wavelength units.
      majtick=20.

c     Plot results to screen first.
      psplot = 0
C     Look for proper screen device
 70   pgstat = pgopen('/XS')
      if (pgstat .le. 0) then
         write(*,*) "/XS device unavailable; trying /AQT."
         pgstat = pgopen('/AQT')
         if (pgstat .le. 0) then
            write(*,*) "/AQT device unavailable; querying."
            pgstat = pgopen('?')
         end if
      end if

C==================
C Read rings files
C==================

 20   if (psplot.eq.0) write (*,'(/,a)')
     $     "Reading in newest spectrum."
      firstspec=1
      write(name,'("ccd",i4.4,".rings")') nimnew
      
 25   open(3,file=name,status='old',iostat=ios)
      if (ios .ne. 0) then
         write(*,'(a,a30)') "Error opening file: ",name
         stop
      end if
      if (firstspec.eq.1) then
         read(3,*)
         read(3,*) wave0,xfine,yfine,zfine,zcoarse
         read(3,*)
         read(3,*) date,time,filter,wavepkmed,fluxpkmed
         read(3,*)
      else
         read(3,*)
         read(3,*) dummy,dummy,dummy,zfineref,dummy
         read(3,*)
         read(3,*) dummy,dummy,filterref,dummy,dummy
         read(3,*)
      end if
C     Sanity check on Z_fine and filter
      if (firstspec.eq.0.AND.psplot.eq.0) then
         if (zfine.ne.zfineref) then
            write(*,'(a,a,$)') "WARNING: Z_fine values are not equal."
     $           ,"  Do you wish to continue? [n]"
            read(*,'(a)') answer
            if (answer.eq.'   ') stop
         end if
         if (filter.ne.filterref) then
            write(*,'(a)') "EXITING: Filters not the same."
            stop
         end if
      end if
C     Get radial spectrum
      nz = 1
 30   read(3,*,iostat=ios) dummy,wave(nz),dummy,flux(nz),dummy,ferr(nz)
C     Check for end-of-file, otherwise keep reading.
      if (ios .ne. 0) then
         nz = nz - 1
         close(3)
         go to 40
      end if
      nz = nz+1
C     Go to next radial bin
      go to 30
 40   continue

C==============
C Plot results
C==============

      if (firstspec.eq.1) then
C     Compute maximum and minimum flux values.
 60      yplotmin = 1.e9
         yplotmax = -1.e9
         do i=1,nz
            yplotmin = min(yplotmin,flux(i))
            yplotmax = max(yplotmax,flux(i))
         end do
c     Set-up.
         call pgscf(2)
         call pgslw(2)
         call pgsvp(0.1,0.9,0.1,0.9)
c     Axis labels
         call pgswin(wave(nz)-wave(1),0,yplotmin,yplotmax)
         call pgbox('BCNST',majtick,10,'BCNST',0,0)
         write(xlabel,'(a,a1,a)') "\gl(R) - \gl(R=0)  [",197,"]"
         call pglabel (xlabel,'Counts / Pixel','')
         call pgswin(wave(nz),wave(1),yplotmin,yplotmax)
      else
         diffpkmed = wavepkmed - newwavepkmed
C     Write top and other labels.
         ilen = index(newname,'.',.true.)-1
         ilen2 = index(name,'.',.true.)-1
         write(toplabel1,'(a,a11,a21,a40)') "UT ",date,time
     $        ,newname(1:ilen)
         write(toplabel2,'(a,i2,a,a27,3(a,i4))') "Z_c=",zcoarse
     $        ,"  Filter: ",filter,"X_f=",xfine,"  Y_f=",yfine ,"  Z_f="
     $        ,zfine
         call pgmtxt('T',1.9,0.5,0.5,toplabel1)
         call pgmtxt('T',0.7,0.5,0.5,toplabel2)
         write(peaklabel0,'(a,f7.2,a,a1)') "\gD\gl = ",-diffpkmed," "
     $        ,197
         write(peaklabel1,'(a,f7.2,a,a1)') "\gDA = ",diffpkmed," ",197
         if (psplot.eq.0) then
            write(peaklabel2,'(a,a10)') "Ref:",name(1:ilen2)
         else
            write(peaklabel2,'(a,a10,a)') "Ref:",name(1:ilen2)
     $           ," (dotted)"
         end if
         call pgmtxt('LV',-0.9,0.92,0.,peaklabel0)
         call pgmtxt('LV',-0.9,0.88,0.,peaklabel1)
         call pgsci(2)
         call pgmtxt('LV',-0.9,0.96,0.,peaklabel2)
      end if
c     Plot data.
      if (psplot.eq.1.AND.firstspec.eq.0) call pgsls(4)
      call pgline (nz,wave,flux)
      call pgsls(1)
C     Plot line through peak
      call pgsls(2)
      plotpkf = (/ 0, 1000000000 /)
      plotpkw = (/ wavepkmed, wavepkmed /)
      call pgline (2, plotpkw, plotpkf)
      call pgsls(1)

      if (firstspec.eq.1) then
         firstspec=0
         newwavepkmed=wavepkmed
         newname=name
         if (psplot.eq.0) write (*,'(a)')
     $        "Reading in reference spectrum."
         write(name,'("ccd",i4.4,".rings")') nimref
         go to 25
      end if

      call pgend()

c     Plot results to file

      if (psplot .eq. 0) then
C     Look for proper plot device
         write(psfile,'(2(a,i4.4),a)') "ringdrift_",nimref,"vs",nimnew
     $        ,".ps"
         pgstat = pgopen(psfile//'/PS')
         if (pgstat .le. 0) then
            write(*,*)
     $           "/PS device unavailable; not producing postscript."
         else
            write(*,'("Writing output plot: ",a30,/)') psfile
            psplot = 1
            go to 20
         end if
      end if

C     Verbose output: note to observer.
      if (diffpkmed.gt.0) then
         radchg="OUTWARD"
         achg="INCREASE"
         zchg="DECREASE"
      else
         radchg="INWARD"
         achg="DECREASE"
         zchg="INCREASE"
      end if
      write(*,'(a,/)') "******** NOTE *********"
      write(*,'(3(a),/,2(a),f6.2,3(a),/,3(a),/)') "The ring has moved "
     $     ,radchg," in radius.  ",achg ," A by " ,abs(diffpkmed)," "
     $     ,197 ," in the online FP calculator."
     $     ,"The value of Z_fine for the wavelength of interest will "
     $     ,zchg,"."


C     THE END
      end
